@@ -1,5 +1,5 @@
 import sys
-from flask import Flask, request, jsonify, render_template, session, redirect, url_for
+from flask import Flask, request, jsonify, render_template, session, redirect, url_for, Response, stream_with_context
 from flask_httpauth import HTTPBasicAuth
 import threading
 import os
@@ -13,7 +13,7 @@ import time
 import shlex
 from gunicorn.app.base import Application
 import ipaddress
-from socket import socket, gethostname, gethostbyname_ex, gethostbyaddr, inet_aton, inet_ntoa, AF_INET, SOCK_STREAM
+from socket import socket, AF_INET, SOCK_STREAM
 import ssl
 import re
 import pwd
@@ -24,7 +24,9 @@ import fcntl
 import termios
 import struct
 import subprocess
-
+import logging
+import pyte
+from host_ip import get_host_ip
 
 if os.environ.get('PYWEBEXEC_LDAP_SERVER'):
     from ldap3 import Server, Connection, ALL, SIMPLE, SUBTREE, Tls
@@ -32,65 +34,42 @@ if os.environ.get('PYWEBEXEC_LDAP_SERVER'):
 app = Flask(__name__)
 app.secret_key = os.urandom(24)  # Secret key for session management
 app.config['SESSION_COOKIE_SAMESITE'] = 'Lax'  # Add SameSite attribute to session cookies
+app.config['SESSION_COOKIE_SECURE'] = True
+app.config['SESSION_COOKIE_HTTPONLY'] = True
 auth = HTTPBasicAuth()
 
 app.config['LDAP_SERVER'] = os.environ.get('PYWEBEXEC_LDAP_SERVER')
-app.config['LDAP_USER_ID'] = os.environ.get('PYWEBEXEC_LDAP_USER_ID', "uid")
+app.config['LDAP_USER_ID'] = os.environ.get('PYWEBEXEC_LDAP_USER_ID', "uid") # sAMAccountName
 app.config['LDAP_GROUPS'] = os.environ.get('PYWEBEXEC_LDAP_GROUPS')
 app.config['LDAP_BASE_DN'] = os.environ.get('PYWEBEXEC_LDAP_BASE_DN')
 app.config['LDAP_BIND_DN'] = os.environ.get('PYWEBEXEC_LDAP_BIND_DN')
 app.config['LDAP_BIND_PASSWORD'] = os.environ.get('PYWEBEXEC_LDAP_BIND_PASSWORD')
 
+# Get the Gunicorn error logger
+gunicorn_logger = logging.getLogger('gunicorn.error')
+app.logger.handlers = gunicorn_logger.handlers
+app.logger.setLevel(logging.INFO)  # Set the logging level to INFO
+
 # Directory to store the command status and output
 CWD = os.getcwd()
+PYWEBEXEC = os.path.abspath(__file__)
 COMMAND_STATUS_DIR = '.web_status'
 CONFDIR = os.path.expanduser("~/").rstrip('/')
 if os.path.isdir(f"{CONFDIR}/.config"):
     CONFDIR += '/.config'
 CONFDIR += "/.pywebexec"
 term_command_id = str(uuid.uuid4())
+tty_cols = 125
+tty_rows = 30
 
 # In-memory cache for command statuses
-command_status_cache = {}
+status_cache = {}
 
 def generate_random_password(length=12):
     characters = string.ascii_letters + string.digits + string.punctuation
     return ''.join(random.choice(characters) for i in range(length))
 
 
-def resolve_hostname(host):
-    """try get fqdn from DNS/hosts"""
-    try:
-        hostinfo = gethostbyname_ex(host)
-        return (hostinfo[0].rstrip('.'), hostinfo[2][0])
-    except OSError:
-        return (host, host)
-
-
-def resolve_ip(ip):
-    """try resolve hostname by reverse dns query on ip addr"""
-    ip = inet_ntoa(inet_aton(ip))
-    try:
-        ipinfo = gethostbyaddr(ip)
-        return (ipinfo[0].rstrip('.'), ipinfo[2][0])
-    except OSError:
-        return (ip, ip)
-
-
-def is_ip(host):
-    """determine if host is valid ip"""
-    try:
-        inet_aton(host)
-        return True
-    except OSError:
-        return False
-
-
-def resolve(host_or_ip):
-    """resolve hostname from ip / hostname"""
-    if is_ip(host_or_ip):
-        return resolve_ip(host_or_ip)
-    return resolve_hostname(host_or_ip)
 
 
 def generate_selfsigned_cert(hostname, ip_addresses=None, key=None):
@@ -103,7 +82,7 @@ def generate_selfsigned_cert(hostname, ip_addresses=None, key=None):
     from cryptography.hazmat.backends import default_backend
     from cryptography.hazmat.primitives import serialization
     from cryptography.hazmat.primitives.asymmetric import rsa
-    
+
     # Generate our key
     if key is None:
         key = rsa.generate_private_key(
@@ -111,16 +90,16 @@ def generate_selfsigned_cert(hostname, ip_addresses=None, key=None):
             key_size=2048,
             backend=default_backend(),
         )
-    
+
     name = x509.Name([
         x509.NameAttribute(NameOID.COMMON_NAME, hostname)
     ])
- 
-    # best practice seem to be to include the hostname in the SAN, which *SHOULD* mean COMMON_NAME is ignored.    
+
+    # best practice seem to be to include the hostname in the SAN, which *SHOULD* mean COMMON_NAME is ignored.
     alt_names = [x509.DNSName(hostname)]
     alt_names.append(x509.DNSName("localhost"))
-    
-    # allow addressing by IP, for when you don't have real DNS (common in most testing scenarios 
+
+    # allow addressing by IP, for when you don't have real DNS (common in most testing scenarios
     if ip_addresses:
         for addr in ip_addresses:
             # openssl wants DNSnames for ips...
@@ -129,7 +108,7 @@ def generate_selfsigned_cert(hostname, ip_addresses=None, key=None):
             # note: older versions of cryptography do not understand ip_address objects
             alt_names.append(x509.IPAddress(ipaddress.ip_address(addr)))
     san = x509.SubjectAlternativeName(alt_names)
-    
+
     # path_len=0 means this cert can only sign itself, not other certs.
     basic_contraints = x509.BasicConstraints(ca=True, path_length=0)
     now = datetime.now(timezone.utc)
@@ -155,7 +134,6 @@ def generate_selfsigned_cert(hostname, ip_addresses=None, key=None):
     return cert_pem, key_pem
 
 
-
 class PyWebExec(Application):
 
     def __init__(self, app, options=None):
@@ -173,9 +151,10 @@ class PyWebExec(Application):
 
     def load(self):
         return self.application
-
-
-ANSI_ESCAPE = re.compile(br'(?:\x1B[@-Z\\-_]|\x1B([(]B|>)|(?:\x1B\[|\x9B)[0-?]*[ -/]*[@-~]|\x1B\[[0-9]{1,2};[0-9]{1,2}[m|K]|\x1B\[[0-9;]*[mGKHF]|[\x00-\x1F\x7F])')
+                                                                                                    #38;2;66;59;165m
+ANSI_ESCAPE = re.compile(br'(?:\x1B[@-Z\\-_]|\x1B([(]B|>)|(?:\x1B\[|\x9B)[0-?]*[ -/]*[@-~]|\x1B\[([0-9]{1,2};){0,4}[0-9]{1,3}[m|K]|\x1B\[[0-9;]*[mGKHF]|[\x00-\x1F\x7F])')
+ANSI_ESCAPE = re.compile(br'(?:\x1B[@-Z\\-_]|\x1B([(]B|>)|(?:\x1B\[|\x9B)[0-?]*[ -/]*[@-~]|\x1B\[([0-9]{1,2};){0,4}[0-9]{1,3}[m|K]|\x1B\[[0-9;]*[mGKHF])')
+ANSI_ESCAPE = re.compile(br'(\x1B\[([0-9]{1,2};){0,4}[0-9]{1,3}[m|K]|\x1B\[[0-9;]*[mGKHF])')
 
 def strip_ansi_control_chars(text):
     """Remove ANSI and control characters from the text."""
@@ -185,56 +164,57 @@ def strip_ansi_control_chars(text):
 def decode_line(line: bytes) -> str:
     """try decode line exception on binary"""
     try:
-        return strip_ansi_control_chars(line).decode().strip(" ")
+        return get_visible_output(line.decode())
     except UnicodeDecodeError:
         return ""
 
 
-def last_line(fd, maxline=1000):
-    """last non empty line of file"""
-    line = "\n"
-    fd.seek(0, os.SEEK_END)
-    size = 0
-    last_pos = 0
-    while line in ["", "\n", "\r"] and size < maxline:
-        try:  # catch if file empty / only empty lines
-            if last_pos:
-                fd.seek(last_pos-2, os.SEEK_SET)
-            while fd.read(1) not in [b"\n", b"\r"]:
-                fd.seek(-2, os.SEEK_CUR)
-                size += 1
+def get_visible_output(line, cols, rows):
+    """pyte vt100 render to get last line"""
+    try:
+        screen = pyte.Screen(cols, rows)
+        stream = pyte.ByteStream(screen)
+        stream.feed(line)
+        visible_line = ""
+        for visible_line in reversed(screen.display):
+            visible_line = visible_line.strip(" ")
+            if visible_line:
+                return visible_line
+    except:
+        return ""
+    return ""
+
+
+def get_last_line(file_path, cols=None, rows=None, maxsize=2048):
+    """Retrieve last non empty line after vt100 interpretation"""
+    cols = cols or tty_cols
+    rows = rows or tty_rows
+    with open(file_path, 'rb') as fd:
+        try:
+            fd.seek(-maxsize, os.SEEK_END)
         except OSError:
             fd.seek(0)
-            line = decode_line(fd.readline())
-            break
-        last_pos = fd.tell()
-        line = decode_line(fd.readline())
-    return line.strip()
+        return get_visible_output(fd.read(), cols, rows)
 
 
-def get_last_non_empty_line_of_file(file_path):
-    """Get the last non-empty line of a file."""
-    with open(file_path, 'rb') as f:
-        return last_line(f)
-    
-
 def start_gunicorn(daemonized=False, baselog=None):
+    check_processes()
     pidfile = f"{baselog}.pid"
     if daemonized:
         if daemon_d('status', pidfilepath=baselog, silent=True):
             print(f"Error: pywebexec already running on {args.listen}:{args.port}", file=sys.stderr)
             return 1
 
-    if sys.stdout.isatty():
+    if daemonized or not sys.stdout.isatty():
+        errorlog = f"{baselog}.log"
+        accesslog = None #f"{baselog}.access.log"
+    else:
         errorlog = "-"
         accesslog = None #"-"
-    else:
-        errorlog = f"{baselog}.log"
-        accesslog = None # f"{baselog}.access.log"
-
     options = {
         'bind': '%s:%s' % (args.listen, args.port),
-        'workers': 4,
+        'workers': 1,
+        'threads': 4,
         'timeout': 600,
         'certfile': args.cert,
         'keyfile': args.key,
@@ -294,7 +274,7 @@ def daemon_d(action, pidfilepath, silent=False, hostname=None, args=None):
 def start_term():
     os.environ["PYWEBEXEC"] = " (shared)"
     os.chdir(CWD)
-    start_time = datetime.now().isoformat()
+    start_time = datetime.now(timezone.utc).isoformat()
     user = pwd.getpwuid(os.getuid())[0]
     print(f"Starting terminal session for {user} : {term_command_id}")
     update_command_status(term_command_id, {
@@ -306,7 +286,7 @@ def start_term():
     })
     output_file_path = get_output_file_path(term_command_id)
     res = script(output_file_path)
-    end_time = datetime.now().isoformat()
+    end_time = datetime.now(timezone.utc).isoformat()
     update_command_status(term_command_id, {
         'status': 'success',
         'end_time': end_time,
@@ -323,11 +303,13 @@ def print_urls(command_id=None):
     if token:
         url_params = f"?token={token}"
     if command_id:
-        print(f"{protocol}://{hostname}:{args.port}/dopopup/{command_id}{url_params}")
-        print(f"{protocol}://{ip}:{args.port}/dopopup/{command_id}{url_params}")
+        print(f"web popup: {protocol}://{hostname}:{args.port}/dopopup/{command_id}{url_params}", flush=True)
+        print(f"web popup: {protocol}://{ip}:{args.port}/dopopup/{command_id}{url_params}", flush=True)
+        print(f"raw output: {protocol}://{hostname}:{args.port}/command_output_raw/{command_id}{url_params}", flush=True)
+        print(f"raw output: {protocol}://{ip}:{args.port}/command_output_raw/{command_id}{url_params}", flush=True)
     else:
-        print(f"{protocol}://{hostname}:{args.port}{url_params}")
-        print(f"{protocol}://{ip}:{args.port}{url_params}")
+        print(f"web commands: {protocol}://{hostname}:{args.port}{url_params}", flush=True)
+        print(f"web commands: {protocol}://{ip}:{args.port}{url_params}", flush=True)
 
 
 def is_port_in_use(address, port):
@@ -383,7 +365,7 @@ def parseargs():
         COMMAND_STATUS_DIR = f"{os.getcwd()}/{COMMAND_STATUS_DIR}"
         sys.exit(start_term())
 
-    (hostname, ip) = resolve(gethostname()) if args.listen == '0.0.0.0' else resolve(args.listen)
+    (hostname, ip) = get_host_ip(args.listen)
 
     if args.tokenurl:
         token = os.environ.get("PYWEBEXEC_TOKEN", token_urlsafe())
@@ -421,39 +403,37 @@ def get_output_file_path(command_id):
 
 def update_command_status(command_id, updates):
     status_file_path = get_status_file_path(command_id)
-    status_data = read_command_status(command_id) or {}
-    status_data.update(updates)
-    if status_data['status'] != 'running':
+    status = read_command_status(command_id) or {}
+    status.update(updates)
+    if status.get('status') != 'running':
         output_file_path = get_output_file_path(command_id)
         if os.path.exists(output_file_path):
-            status_data['last_output_line'] = get_last_non_empty_line_of_file(output_file_path)
+            status['last_output_line'] = get_last_line(output_file_path, status.get('cols'), status.get('rows'))
+    status_cache[command_id] = status
     with open(status_file_path, 'w') as f:
-        json.dump(status_data, f)
-    
-    # Update cache if status is not "running"
-    if status_data['status'] != 'running':
-        command_status_cache[command_id] = status_data
-    elif command_id in command_status_cache:
-        del command_status_cache[command_id]
+        json.dump(status, f)
+
 
 def read_command_status(command_id):
     # Return cached status if available
-    if command_id in command_status_cache:
-        return command_status_cache[command_id]
-    
+    status_data = {}
+    if command_id in status_cache:
+        status_data = status_cache[command_id]
+    status = status_data.get('status')
+    if status and status != "running":
+        return status_data
+    if command_id in status_cache and status_cache.get('last_read',0)>datetime.now().timestamp()-0.5:
+        return status_data
     status_file_path = get_status_file_path(command_id)
     if not os.path.exists(status_file_path):
         return None
     with open(status_file_path, 'r') as f:
         try:
-            status_data = json.load(f)
+            status_data.update(json.load(f))
         except json.JSONDecodeError:
             return None
-    
-    # Cache the status if it is not "running"
-    if status_data['status'] != 'running':
-        command_status_cache[command_id] = status_data
-    
+    status_data['last_read'] = datetime.now().timestamp()
+    status_cache[command_id] = status_data
     return status_data
 
 def sigwinch_passthrough(sig, data):
@@ -473,36 +453,38 @@ def script(output_file):
     with open(output_file, 'wb') as fd:
         p = pexpect.spawn(shell, echo=True)
         p.logfile_read = fd
+        update_command_status(term_command_id, {"pid": p.pid})
         # Set the window size
         sigwinch_passthrough(None, None)
         signal.signal(signal.SIGWINCH, sigwinch_passthrough)
         p.interact()
-    
 
-def run_command(command, params, command_id, user):
-    start_time = datetime.now().isoformat()
+
+def run_command(fromip, user, command, params, command_id, rows, cols):
+    # app.logger.info(f'{fromip} run_command {command_id} {user}: {command} {params}')
+    log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}')
+    start_time = datetime.now(timezone.utc).isoformat()
     update_command_status(command_id, {
         'status': 'running',
         'command': command,
         'params': params,
         'start_time': start_time,
-        'user': user
+        'user': user,
+        'cols': cols,
+        'rows': rows,
     })
     output_file_path = get_output_file_path(command_id)
     try:
         with open(output_file_path, 'wb') as fd:
-            p = pexpect.spawn(command, params, ignore_sighup=True, timeout=None)
+            p = pexpect.spawn(command, params, ignore_sighup=True, timeout=None, dimensions=(rows, cols))
             update_command_status(command_id, {
-                'status': 'running',
                 'pid': p.pid,
-                'user': user
             })
-            p.setwinsize(24, 125)
             p.logfile = fd
             p.expect(pexpect.EOF)
             fd.flush()
             status = p.wait()
-            end_time = datetime.now().isoformat()
+            end_time = datetime.now(timezone.utc).isoformat()
             # Update the status based on the result
             if status is None:
                 exit_code = -15
@@ -512,6 +494,7 @@ def run_command(command, params, command_id, user):
                     'exit_code': exit_code,
                     'user': user
                 })
+                log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}: command aborted')
             else:
                 exit_code = status
                 if exit_code == 0:
@@ -521,6 +504,7 @@ def run_command(command, params, command_id, user):
                         'exit_code': exit_code,
                         'user': user
                     })
+                    log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}: completed successfully')
                 else:
                     update_command_status(command_id, {
                         'status': 'failed',
@@ -528,8 +512,10 @@ def run_command(command, params, command_id, user):
                         'exit_code': exit_code,
                         'user': user
                     })
+                    log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}: exit code {exit_code}')
+
     except Exception as e:
-        end_time = datetime.now().isoformat()
+        end_time = datetime.now(timezone.utc).isoformat()
         update_command_status(command_id, {
             'status': 'failed',
             'end_time': end_time,
@@ -538,32 +524,104 @@ def run_command(command, params, command_id, user):
         })
         with open(get_output_file_path(command_id), 'a') as output_file:
             output_file.write(str(e))
+        app.logger.error(fromip, user, f'Error running command {command_id}: {e}')
+
+
+def command_str(command, params):
+    try:
+        params = shlex.join(params)
+    except AttributeError:
+        params = " ".join([shlex.quote(p) if " " in p else p for p in params])
+    return f"{command} {params}"
+
+
+def read_commands():
+    global status_cache
+    commands = []
+    for filename in os.listdir(COMMAND_STATUS_DIR):
+        if filename.endswith('.json'):
+            command_id = filename[:-5]
+            status = read_command_status(command_id)
+            if status:
+                command = command_str(status.get('command', '-'), status.get('params', []))
+                if status.get('status') == 'running' and status.get('last_update',0)<datetime.now().timestamp()-5:
+                    output_file_path = get_output_file_path(command_id)
+                    if os.path.exists(output_file_path):
+                        size = os.path.getsize(output_file_path)
+                        if size != status.get('size'):
+                            status.update({
+                                'size': size,
+                                'last_update': datetime.now().timestamp(),
+                                'last_output_line': get_last_line(output_file_path, status.get('cols'), status.get('rows')),
+                            })
+                            status_cache[command_id] = status
+                commands.append({
+                    'command_id': command_id,
+                    'status': status.get('status'),
+                    'start_time': status.get('start_time', 'N/A'),
+                    'end_time': status.get('end_time', 'N/A'),
+                    'command': command,
+                    'user': status.get('user'),
+                    'exit_code': status.get('exit_code', 'N/A'),
+                    'last_output_line': status.get('last_output_line'),
+                })
+    return commands
+
+def is_process_alive(pid):
+    try:
+        os.kill(pid, 0)
+    except ProcessLookupError:
+        return False
+    except PermissionError:
+        return True
+    return True
+
+def check_processes():
+    for filename in os.listdir(COMMAND_STATUS_DIR):
+        if filename.endswith('.json'):
+            command_id = filename[:-5]
+            status = read_command_status(command_id)
+            if status.get('status') == 'running' and 'pid' in status:
+                if not is_process_alive(status['pid']):
+                    end_time = datetime.now(timezone.utc).isoformat()
+                    update_command_status(command_id, {
+                        'status': 'aborted',
+                        'end_time': end_time,
+                        'exit_code': -1,
+                    })
+
+parseargs()
+
+def log_info(fromip, user, message):
+    app.logger.info(f"{user} {fromip}: {message}")
+
+def log_error(fromip, user, message):
+    app.logger.error(f"{user} {fromip}: {message}")
+
+def log_request(message):
+    log_info(request.remote_addr, session.get('username', '-'), message)
 
 @app.route('/stop_command/<command_id>', methods=['POST'])
 def stop_command(command_id):
+    log_request(f"stop_command {command_id}")
     status = read_command_status(command_id)
     if not status or 'pid' not in status:
         return jsonify({'error': 'Invalid command_id or command not running'}), 400
 
     pid = status['pid']
-    end_time = datetime.now().isoformat()
+    end_time = datetime.now(timezone.utc).isoformat()
     try:
         #update_command_status(command_id, 'aborted', end_time=end_time, exit_code=-15)
         os.killpg(os.getpgid(pid), 15)  # Send SIGTERM to the process group
         return jsonify({'message': 'Command aborted'})
     except Exception as e:
-        status_data = read_command_status(command_id) or {}
-        status_data['status'] = 'failed'
-        status_data['end_time'] = end_time
-        status_data['exit_code'] = 1
-        with open(get_status_file_path(command_id), 'w') as f:
-            json.dump(status_data, f)
-        with open(get_output_file_path(command_id), 'a') as output_file:
-            output_file.write(str(e))
+        update_command_status(command_id, {
+            'status': 'aborted',
+            'end_time': end_time,
+            'exit_code': -15,
+        })
         return jsonify({'error': 'Failed to terminate command'}), 500
 
-parseargs()
-
 
 @app.before_request
 def check_authentication():
@@ -573,7 +631,7 @@ def check_authentication():
         if request.args.get('token') == token:
             return
         return jsonify({'error': 'Forbidden'}), 403
-    
+
     if not app.config['USER'] and not app.config['LDAP_SERVER']:
         return
 
@@ -598,30 +656,30 @@ def verify_ldap(username, password):
     tls_configuration = Tls(validate=ssl.CERT_NONE, version=ssl.PROTOCOL_TLSv1_2) if app.config['LDAP_SERVER'].startswith("ldaps:") else None
     server = Server(app.config['LDAP_SERVER'], tls=tls_configuration, get_info=ALL)
     user_filter = f"({app.config['LDAP_USER_ID']}={username})"
+    group_filter = ""
+    if app.config["LDAP_GROUPS"]:
+        group_filter = "".join(f"(memberOf={group})" for group in app.config['LDAP_GROUPS'].split(" "))
+        group_filter = f"(|{group_filter})"
+    ldap_filter = f"(&(objectClass=person){user_filter}{group_filter})"
     try:
         # Bind with the bind DN and password
         conn = Connection(server, user=app.config['LDAP_BIND_DN'], password=app.config['LDAP_BIND_PASSWORD'], authentication=SIMPLE, auto_bind=True, read_only=True)
         try:
-            conn.search(search_base=app.config['LDAP_BASE_DN'], search_filter=user_filter, search_scope=SUBTREE)
+            conn.search(search_base=app.config['LDAP_BASE_DN'], search_filter=ldap_filter, search_scope=SUBTREE)
             if len(conn.entries) == 0:
-                print(f"User {username} not found in LDAP.")
+                print(f"User {username} not found in LDAP in allowed groups.")
                 return False
             user_dn = conn.entries[0].entry_dn
         finally:
             conn.unbind()
-        
+
         # Bind with the user DN and password to verify credentials
         conn = Connection(server, user=user_dn, password=password, authentication=SIMPLE, auto_bind=True, read_only=True)
         try:
-            if not app.config['LDAP_GROUPS'] and conn.result["result"] == 0:
+            if conn.result["result"] == 0:
                 return True
-            group_filter = "".join([f'({group})' for group in app.config['LDAP_GROUPS'].split(",")])
-            group_filter = f"(&{group_filter}(|(member={user_dn})(uniqueMember={user_dn})))"
-            conn.search(search_base=app.config['LDAP_BASE_DN'], search_filter=group_filter, search_scope=SUBTREE)
-            result = len(conn.entries) > 0
-            if not result:
-                print(f"User {username} is not a member of groups {app.config['LDAP_GROUPS']}.")
-            return result
+            print(f"{username}: Password mismatch")
+            return False
         finally:
             conn.unbind()
     except Exception as e:
@@ -633,6 +691,8 @@ def run_command_endpoint():
     data = request.json
     command = data.get('command')
     params = data.get('params', [])
+    rows = data.get('rows', tty_rows)
+    cols = data.get('cols', tty_cols)
 
     if not command:
         return jsonify({'error': 'command is required'}), 400
@@ -656,11 +716,12 @@ def run_command_endpoint():
         'status': 'running',
         'command': command,
         'params': params,
-        'user': user
+        'user': user,
+        'from': request.remote_addr,
     })
 
     # Run the command in a separate thread
-    thread = threading.Thread(target=run_command, args=(command_path, params, command_id, user))
+    thread = threading.Thread(target=run_command, args=(request.remote_addr, user, command_path, params, command_id, rows, cols))
     thread.start()
 
     return jsonify({'message': 'Command is running', 'command_id': command_id})
@@ -685,32 +746,8 @@ def index():
 
 @app.route('/commands', methods=['GET'])
 def list_commands():
-    commands = []
-    for filename in os.listdir(COMMAND_STATUS_DIR):
-        if filename.endswith('.json'):
-            command_id = filename[:-5]
-            status = read_command_status(command_id)
-            if status:
-                try:
-                    params = shlex.join(status.get('params', []))
-                except AttributeError:
-                    params = " ".join([shlex.quote(p) if " " in p else p for p in status['params']])
-                command = status.get('command', '-') + ' ' + params
-                last_line = status.get('last_output_line')
-                if last_line is None:
-                    output_file_path = get_output_file_path(command_id)
-                    if os.path.exists(output_file_path):
-                        last_line = get_last_non_empty_line_of_file(output_file_path)
-                commands.append({
-                    'command_id': command_id,
-                    'status': status['status'],
-                    'start_time': status.get('start_time', 'N/A'),
-                    'end_time': status.get('end_time', 'N/A'),
-                    'command': command,
-                    'exit_code': status.get('exit_code', 'N/A'),
-                    'last_output_line': last_line,
-                })
     # Sort commands by start_time in descending order
+    commands = read_commands()
     commands.sort(key=lambda x: x['start_time'], reverse=True)
     return jsonify(commands)
 
@@ -731,6 +768,7 @@ def get_command_output(command_id):
             'output': output[-maxsize:],
             'status': status_data.get("status"),
             'cols': status_data.get("cols"),
+            'rows': status_data.get("rows"),
             'links': {
                 'next': f'{request.url_root}command_output/{command_id}?offset={new_offset}&maxsize={maxsize}{token_param}'
             }
@@ -740,6 +778,30 @@ def get_command_output(command_id):
         return jsonify(response)
     return jsonify({'error': 'Invalid command_id'}), 404
 
+@app.route('/command_output_raw/<command_id>', methods=['GET'])
+def get_command_output_raw(command_id):
+    offset = int(request.args.get('offset', 0))
+    @stream_with_context
+    def generate(offset):
+        try:
+            output_file_path = get_output_file_path(command_id)
+            if os.path.exists(output_file_path):
+                with open(output_file_path, 'rb') as output_file:
+                    while True:
+                        while True:
+                            chunk = output_file.read(1024)
+                            if not chunk:
+                                time.sleep(0.5)
+                                break
+                            yield chunk
+                        status = read_command_status(command_id)
+                        if not status or status['status'] != 'running':
+                            yield f"\nEnd of command {command_id} {status.get('status', '')} exit: {status.get('exit_code')}\n"
+                            break
+        except GeneratorExit:
+            return
+    return Response(generate(offset), content_type='text/plain')
+
 @app.route('/executables', methods=['GET'])
 def list_executables():
     executables = [f for f in os.listdir('.') if os.path.isfile(f) and os.access(f, os.X_OK)]
@@ -777,16 +839,18 @@ def main():
         args.action = "start"
     port_used = is_port_in_use(args.listen, args.port)
     if args.action != "stop":
-        print("Starting server:")
+        print("Starting server:", flush=True)
         print_urls()
     if args.action != "stop" and port_used:
         print(f"Error: port {args.port} already in use", file=sys.stderr)
         return 1
     if args.action == "shareterm":
         COMMAND_STATUS_DIR = f"{os.getcwd()}/{COMMAND_STATUS_DIR}"
+        check_processes()
         sys.argv.remove("shareterm")
-        with open(basef + ".log", "ab+") as log:
-            pywebexec = subprocess.Popen([sys.executable] + sys.argv, stdout=log, stderr=log)
+        sys.argv[0] = PYWEBEXEC
+        with open(basef + ".log", "a") as log:
+            pywebexec = subprocess.Popen([sys.executable] + sys.argv, stdout=log, stderr=log, bufsize=1)
             print_urls(term_command_id)
             res = start_term()
             print("Stopping server")
