@@ -55,6 +55,18 @@ const fitAddon = new FitAddon.FitAddon();
 terminal.loadAddon(fitAddon);
 terminal.open(document.getElementById('output'));
 fitAddon.fit();
+terminal.onTitleChange((title) => {
+    document.getElementById('commandInfo').innerText = title;
+})
+terminal.onSelectionChange(() => {
+    const selectionText = terminal.getSelection();
+    if (selectionText) {
+        navigator.clipboard.writeText(selectionText).catch(err => {
+            console.error('Failed to copy text to clipboard:', err);
+        });
+    }
+});
+
 
 let currentCommandId = null;
 let outputInterval = null;
@@ -63,8 +75,30 @@ let fullOutput = '';
 let outputLength = 0;
 let slider = null;
 let isPaused = false;
+let cols = 0;
+let rows = 0;
+let fitWindow = false;
+
 const toggleButton = document.getElementById('toggleFetch');
 const pausedMessage = document.getElementById('pausedMessage');
+const toggleFitButton = document.getElementById('toggleFit');
+
+function autoFit(scroll=true) {
+    // Scroll output div to bottom
+    const outputDiv = document.getElementById('output');
+    outputDiv.scrollTop = terminal.element.clientHeight - outputDiv.clientHeight + 20;
+    if (cols && !fitWindow) {
+        let fit = fitAddon.proposeDimensions();
+        if (fit.rows < rows) {
+            terminal.resize(cols, rows);
+        } else {
+            terminal.resize(cols, fit.rows);
+        }
+    } else {
+        fitAddon.fit();
+    }
+    if (scroll) terminal.scrollToBottom();
+}
 
 function getTokenParam() {
     const urlParams = new URLSearchParams(window.location.search);
@@ -72,20 +106,18 @@ function getTokenParam() {
 }
 const urlToken = getTokenParam();
 
-function extractTitle(output) {
-    const titleindex = output.lastIndexOf('\x1b]0;');
-    if (titleindex < 0) return null;
-    const endindex = output.slice(titleindex+4).indexOf('\x07');
-    if (endindex < 0) return null;
-    return output.slice(titleindex+4, titleindex+4+endindex);
+
+function setCommandStatus(status) {
+    document.getElementById("commandStatus").className = `status-icon status-${status}`;
 }
 
+
 async function fetchOutput(url) {
     if (isPaused) return;
     try {
         const response = await fetch(url);
         if (!response.ok) {
-            document.getElementById('dimmer').style.display = 'none';
+            document.getElementById('dimmer').style.display = 'block';
             return;
         }
         const data = await response.json();
@@ -94,34 +126,32 @@ async function fetchOutput(url) {
             clearInterval(outputInterval);
         } else {
             if (data.cols) {
-                terminal.resize(data.cols, terminal.rows);
-            } else fitAddon.fit();
+                cols = data.cols;
+                rows = data.rows;
+                autoFit(false);
+            }
             percentage = slider.value;
             fullOutput += data.output;
             if (fullOutput.length > maxSize)
                 fullOutput = fullOutput.slice(-maxSize);
-            if (percentage == 100)
+            if (percentage == 1000)
                 terminal.write(data.output);
             else {
-                percentage = Math.round((outputLength * 100)/fullOutput.length);
+                percentage = Math.round((outputLength * 1000)/fullOutput.length);
                 slider.value = percentage;
-                document.getElementById('outputPercentage').innerText = `${percentage}%`;
+                document.getElementById('outputPercentage').innerText = `${Math.floor(percentage/10)}%`;
             }
             nextOutputLink = data.links.next;
             if (data.status != 'running') {
                 clearInterval(outputInterval);
                 document.title = document.title.replace('[running]',`[${data.status}]`);
                 toggleButton.style.display = 'none';
-                document.getElementById('commandStatus').classList.remove('status-running');
-                document.getElementById('commandStatus').classList.add(`status-${data.status}`);
+                setCommandStatus(data.status)
             } else {
                 toggleButton.style.display = 'block';
-                const title = extractTitle(data.output);
-                if (title) {
-                    document.title = `${title} - [running]`;
-                    document.getElementById('commandInfo').innerHTML = `<span id="commandStatus" class="status-icon status-running"></span>${title}`;
-                }
+                setCommandStatus(data.status)
             }
+            document.getElementById('dimmer').style.display = 'none';
         }
     } catch (error) {
         document.getElementById('dimmer').style.display = 'block';
@@ -130,7 +160,7 @@ async function fetchOutput(url) {
 }
 
 async function viewOutput(command_id) {
-    slider.value = 100;
+    slider.value = 1000;
     adjustOutputHeight();
     currentCommandId = command_id;
     nextOutputLink = `/command_output/${command_id}${urlToken}`;
@@ -146,7 +176,8 @@ async function viewOutput(command_id) {
         const data = await response.json();
         const commandInfo = document.getElementById('commandInfo');
         const command = `${data.command.replace(/^\.\//, '')} ${data.params.join(' ')}`;
-        commandInfo.innerHTML = `<span id="commandStatus" class="status-icon status-${data.status}"></span>${command}`;
+        setCommandStatus(data.status);
+        commandInfo.innerText = command;
         commandInfo.setAttribute('title', command);
         document.title = `${data.command} ${data.params.join(' ')} - [${data.status}]`;
         if (data.command == 'term')
@@ -172,23 +203,22 @@ function adjustOutputHeight() {
     const outputTop = outputDiv.getBoundingClientRect().top;
     const maxHeight = windowHeight - outputTop - 60; // Adjusted for slider height
     outputDiv.style.height = `${maxHeight}px`;
-    fitAddon.fit();
-    sliderUpdateOutput();
+    autoFit();
 }
 
 function sliderUpdateOutput() {
-    const percentage = slider.value;
+    const percentage = slider.value / 10;
     outputLength = Math.floor((fullOutput.length * percentage) / 100);
     const limitedOutput = fullOutput.slice(0, outputLength);
     terminal.clear();
     terminal.reset();
     terminal.write(limitedOutput);
-    document.getElementById('outputPercentage').innerText = `${percentage}%`;
+    document.getElementById('outputPercentage').innerText = `${Math.floor(percentage)}%`;
 }
 
 function toggleFetchOutput() {
     if (isPaused) {
-        slider.value = 100;
+        slider.value = 1000;
         document.getElementById('outputPercentage').innerText = '100%';
         terminal.clear();
         terminal.reset();
@@ -208,13 +238,26 @@ function toggleFetchOutput() {
     }
     isPaused = !isPaused;
 }
+function toggleFit() {
+    fitWindow = ! fitWindow;
+    if (fitWindow) {
+        toggleFitButton.classList.add('fit-tty');
+        toggleFitButton.setAttribute('title', 'terminal fit tty');
+    } else {
+        toggleFitButton.classList.remove('fit-tty');
+        toggleFitButton.setAttribute('title', 'terminal fit window');
+    }
+    autoFit();
+    viewOutput(currentCommandId);
+}
 
 toggleButton.addEventListener('click', toggleFetchOutput);
-
+toggleFitButton.addEventListener('click', toggleFit);
 window.addEventListener('resize', adjustOutputHeight);
 window.addEventListener('load', () => {
     slider = document.getElementById('outputSlider');
     slider.addEventListener('input', sliderUpdateOutput);
+    adjustOutputHeight();
     const commandId = window.location.pathname.split('/').slice(-1)[0];
     viewOutput(commandId);
 });
@@ -222,11 +265,11 @@ window.addEventListener('load', () => {
 document.getElementById('decreaseFontSize').addEventListener('click', () => {
     fontSize = Math.max(8, fontSize - 1);
     terminal.options.fontSize = fontSize;
-    fitAddon.fit();
+    autoFit();
 });
 
 document.getElementById('increaseFontSize').addEventListener('click', () => {
     fontSize = Math.min(32, fontSize + 1);
     terminal.options.fontSize = fontSize;
-    fitAddon.fit();
+    autoFit();
 });
