@@ -9,6 +9,11 @@ const maxScrollback = 99999;
 const maxSize = 10485760; // 10MB
 let fontSize = 14;
 let isPaused = false;
+let showRunningOnly = false;
+let hiddenCommandIds = [];
+let fitWindow = false;
+let cols = 0;
+let rows = 0;
 
 function initTerminal()
 {
@@ -16,7 +21,7 @@ function initTerminal()
         cursorBlink: false,
         cursorInactiveStyle: 'none',
         disableStdin: true,
-        convertEol: true,
+        //convertEol: true,
         fontFamily: '"Consolas NF", "Fira Code", monospace, "Powerline Extra Symbols", courier-new, courier',
         fontSize: fontSize,
         scrollback: maxScrollback,
@@ -74,11 +79,17 @@ terminal.loadAddon(fitAddon);
 terminal.open(document.getElementById('output'));
 fitAddon.fit();
 
-function getTokenParam() {
-    const urlParams = new URLSearchParams(window.location.search);
-    return urlParams.get('token') ? `?token=${urlParams.get('token')}` : '';
-}
-const urlToken = getTokenParam();
+terminal.onTitleChange((title) => {
+    document.getElementById('commandInfo').innerText = title;
+})
+
+/*terminal.onResize((evt) => {
+    const terminal_size = {
+        Width: evt.cols,
+        Height: evt.rows,
+    };
+    console.log(terminal_size);
+})*/
 
 terminal.onSelectionChange(() => {
     const selectionText = terminal.getSelection();
@@ -89,25 +100,51 @@ terminal.onSelectionChange(() => {
     }
 });
 
+function autoFit(scroll=true) {
+    // Scroll output div to bottom
+    const outputDiv = document.getElementById('output');
+    outputDiv.scrollTop = terminal.element.clientHeight - outputDiv.clientHeight + 20;
+    if (cols && !fitWindow) {
+        let fit = fitAddon.proposeDimensions();
+        if (fit.rows < rows) {
+            terminal.resize(cols, rows);
+        } else {
+            terminal.resize(cols, fit.rows);
+        }
+    } else {
+        fitAddon.fit();
+    }
+    if (scroll) terminal.scrollToBottom();
+}
+
+function getTokenParam() {
+    const urlParams = new URLSearchParams(window.location.search);
+    return urlParams.get('token') ? `?token=${urlParams.get('token')}` : '';
+}
+const urlToken = getTokenParam();
+
+
 document.getElementById('launchForm').addEventListener('submit', async (event) => {
     event.preventDefault();
-    const commandName = document.getElementById('commandName').value;
-    const params = document.getElementById('params').value.split(' ');
+    const commandName = commandInput.value;
+    const params = paramsInput.value.split(' ');
+    fitAddon.fit();
+    terminal.clear();
     try {
         const response = await fetch(`/run_command${urlToken}`, {
             method: 'POST',
             headers: {
                 'Content-Type': 'application/json'
             },
-            body: JSON.stringify({ command: commandName, params: params })
+            body: JSON.stringify({ command: commandName, params: params, rows: terminal.rows, cols: terminal.cols })
         });
         if (!response.ok) {
             throw new Error('Failed to launch command');
         }
         const data = await response.json();
-        await new Promise(r => setTimeout(r, 300));
-        fetchCommands();
+        //await new Promise(r => setTimeout(r, 300));// not ok
         viewOutput(data.command_id);
+        fetchCommands();
         commandInput.focus()
         commandInput.setSelectionRange(0, commandInput.value.length)
     } catch (error) {
@@ -115,7 +152,7 @@ document.getElementById('launchForm').addEventListener('submit', async (event) =
     }
 });
 
-async function fetchCommands() {
+async function fetchCommands(hide=false) {
     try {
         const response = await fetch(`/commands${urlToken}`);
         if (!response.ok) {
@@ -130,21 +167,28 @@ async function fetchCommands() {
             currentCommandId = commands[0].command_id;
             viewOutput(currentCommandId);
         }
+        runningCommands = [];
         commands.forEach(command => {
+            if (hide && showRunningOnly && command.status !== 'running') {
+                hiddenCommandIds.push(command.command_id);
+                return;
+            }
+            if (hiddenCommandIds.includes(command.command_id)) return;
             const commandRow = document.createElement('tr');
             commandRow.className = `clickable-row ${command.command_id === currentCommandId ? 'currentcommand' : ''}`;
             commandRow.onclick = () => viewOutput(command.command_id);
+            if (command.status === 'running') runningCommands.push(command.command.replace(/^\.\//, ''));
             commandRow.innerHTML = `
                 <td class="monospace">
                     ${navigator.clipboard == undefined ? `${command.command_id.slice(0, 8)}` : `<span class="copy_clip" onclick="copyToClipboard('${command.command_id}', this, event)">${command.command_id.slice(0, 8)}</span>`}
                 </td>
                 <td>${formatTime(command.start_time)}</td>
                 <td>${command.status === 'running' ? formatDuration(command.start_time, new Date().toISOString()) : formatDuration(command.start_time, command.end_time)}</td>
-                <td>${command.command.replace(/^\.\//, '')}</td>
                 <td><span class="status-icon status-${command.status}"></span>${command.status}${command.status === 'failed' ? ` (${command.exit_code})` : ''}</td>
                 <td>
                     ${command.command.startsWith('term') ? '' : command.status === 'running' ? `<button onclick="stopCommand('${command.command_id}', event)">Stop</button>` : `<button onclick="relaunchCommand('${command.command_id}', event)">Run</button>`}
                 </td>
+                <td class="system-font" title="${command.user == '-' ? '' : command.user}">${command.command.replace(/^\.\//, '')}</td>
                 <td class="monospace outcol">
                     <button class="popup-button" onclick="openPopup('${command.command_id}', event)"></button>
                     ${command.last_output_line || ''}
@@ -152,6 +196,13 @@ async function fetchCommands() {
             `;
             commandsTbody.appendChild(commandRow);
         });
+        if (runningCommands.length) {
+            document.getElementById('thStatus').innerHTML=`<span class="status-icon status-running"></span>Running <span class="system-font nbrunning">${runningCommands.length}</span>`;
+            document.getElementById('thStatus').setAttribute('title', runningCommands.join("\n"));
+        } else {
+            document.getElementById('thStatus').innerHTML=`<span class="status-icon status-norun"></span>Status`;
+            document.getElementById('thStatus').setAttribute('title', "no command running");
+        }
         document.getElementById('dimmer').style.display = 'none';
     } catch (error) {
         console.log('Error fetching commands:', error);
@@ -159,14 +210,6 @@ async function fetchCommands() {
     }
 }
 
-function extractTitle(output) {
-    const titleindex = output.lastIndexOf('\x1b]0;');
-    if (titleindex < 0) return null;
-    const endindex = output.slice(titleindex+4).indexOf('\x07');
-    if (endindex < 0) return null;
-    return output.slice(titleindex+4, titleindex+4+endindex);
-}
-
 async function fetchOutput(url) {
     if (isPaused) return;
     try {
@@ -180,33 +223,29 @@ async function fetchOutput(url) {
             clearInterval(outputInterval);
         } else {
             if (data.cols) {
-                terminal.resize(data.cols, terminal.rows);
-            } else fitAddon.fit();
+                cols = data.cols;
+                rows = data.rows;
+                autoFit(scroll=false);
+            }
             fullOutput += data.output;
             if (fullOutput.length > maxSize)
                 fullOutput = fullOutput.slice(-maxSize);
-            if (slider.value == 100)
-                terminal.write(data.output); //.replace(/ \r/g, "\r\n")); tty size mismatch
+            if (slider.value == 1000)
+                terminal.write(data.output);
             else {
-                percentage = Math.round((outputLength * 100)/fullOutput.length);
+                percentage = Math.round((outputLength * 1000)/fullOutput.length);
                 slider.value = percentage;
-                outputPercentage.innerText = `${percentage}%`;
+                outputPercentage.innerText = `${Math.floor(percentage/10)}%`;
             }
             nextOutputLink = data.links.next;
             if (data.status != 'running') {
                 clearInterval(outputInterval);
                 toggleButton.style.display = 'none';
-                document.getElementById('commandStatus').classList.remove('status-running');
-                document.getElementById('commandStatus').classList.add(`status-${data.status}`);
-                const row = document.getElementsByClassName("currentcommand");
-                if (row.length > 0)
-                    row[0].cells[4].innerHTML = `<span class="status-icon status-${data.status}"></span>${data.status}${data.status === 'failed' ? ` (${data.exit_code})` : ''}`;
+                setCommandStatus(data.status);
+                fetchCommands();
             } else {
                 toggleButton.style.display = 'block';
-                const title = extractTitle(data.output);
-                if (title) {
-                    document.getElementById('commandInfo').innerHTML = `<span id="commandStatus" class="status-icon status-running"></span>${title}`;
-                }
+                setCommandStatus(data.status);
             }
         }
     } catch (error) {
@@ -214,8 +253,12 @@ async function fetchOutput(url) {
     }
 }
 
+function setCommandStatus(status) {
+    document.getElementById("commandStatus").className = `status-icon status-${status}`;
+}
+
 async function viewOutput(command_id) {
-    slider.value = 100;
+    slider.value = 1000;
     outputPercentage.innerText = '100%';
     adjustOutputHeight();
     currentCommandId = command_id;
@@ -232,7 +275,8 @@ async function viewOutput(command_id) {
         const data = await response.json();
         const commandInfo = document.getElementById('commandInfo');
         const command = `${data.command.replace(/^\.\//, '')} ${data.params.join(' ')}`;
-        commandInfo.innerHTML = `<span id="commandStatus" class="status-icon status-${data.status}"></span>${command}`;
+        setCommandStatus(data.status)
+        commandInfo.innerText = command;
         commandInfo.setAttribute('title', command);
         if (data.command == 'term')
             terminal.options.cursorInactiveStyle = 'outline';
@@ -272,6 +316,8 @@ async function relaunchCommand(command_id, event) {
             alert(data.error);
             return;
         }
+        fitAddon.fit();
+        terminal.clear();
         const relaunchResponse = await fetch(`/run_command${urlToken}`, {
             method: 'POST',
             headers: {
@@ -279,15 +325,17 @@ async function relaunchCommand(command_id, event) {
             },
             body: JSON.stringify({
                 command: data.command,
-                params: data.params
+                params: data.params,
+                rows: terminal.rows,
+                cols: terminal.cols,
             })
         });
         if (!relaunchResponse.ok) {
             throw new Error('Failed to relaunch command');
         }
         const relaunchData = await relaunchResponse.json();
-        fetchCommands();
         viewOutput(relaunchData.command_id);
+        fetchCommands();
     } catch (error) {
         console.log('Error relaunching command:', error);
         alert('Failed to relaunch command. Please try again.');
@@ -312,14 +360,14 @@ async function stopCommand(command_id, event) {
         }
     } catch (error) {
         console.log('Error stopping command:', error);
-        alert('Failed to stop command. Please try again.');
+        alert('Failed to stop command. Process not found.');
     }
 }
 
 function formatTime(time) {
     if (!time || time === 'N/A') return 'N/A';
     const date = new Date(time);
-    return date.toISOString().slice(0, 16).replace('T', ' ');
+    return date.toLocaleString("sv-SE", { hour12: false, timeStyle: 'short', dateStyle: 'short' }).slice(5);
 }
 
 function formatDuration(startTime, endTime) {
@@ -328,8 +376,8 @@ function formatDuration(startTime, endTime) {
     const end = new Date(endTime);
     const duration = (end - start) / 1000;
     const hours = Math.floor(duration / 3600);
-    const minutes = Math.floor((duration % 3600) / 60);
-    const seconds = Math.floor(duration % 60);
+    const minutes = Math.floor((duration % 3600) / 60).toString().padStart(2, '0');
+    const seconds = Math.floor(duration % 60).toString().padStart(2, '0');
     return `${hours}h ${minutes}m ${seconds}s`;
 }
 
@@ -350,7 +398,7 @@ function adjustOutputHeight() {
     const outputTop = outputDiv.getBoundingClientRect().top;
     const maxHeight = windowHeight - outputTop - 60; // Adjusted for slider height
     outputDiv.style.height = `${maxHeight}px`;
-    fitAddon.fit();
+    autoFit();
 }
 
 function initResizer() {
@@ -378,13 +426,13 @@ function initResizer() {
 
 function sliderUpdateOutput()
 {
-    const percentage = slider.value;
+    const percentage = slider.value/10;
     outputLength = Math.floor((fullOutput.length * percentage) / 100);
     const limitedOutput = fullOutput.slice(0, outputLength);
     terminal.clear();
     terminal.reset();
     terminal.write(limitedOutput);
-    outputPercentage.innerText = `${percentage}%`;
+    outputPercentage.innerText = `${Math.floor(percentage)}%`;
 }
 
 slider.addEventListener('input', sliderUpdateOutput);
@@ -392,17 +440,19 @@ slider.addEventListener('input', sliderUpdateOutput);
 document.getElementById('decreaseFontSize').addEventListener('click', () => {
     fontSize = Math.max(8, fontSize - 1);
     terminal.options.fontSize = fontSize;
-    fitAddon.fit();
+    autoFit();
 });
 
 document.getElementById('increaseFontSize').addEventListener('click', () => {
     fontSize = Math.min(32, fontSize + 1);
     terminal.options.fontSize = fontSize;
-    fitAddon.fit();
+    autoFit();
 });
 
 const toggleButton = document.getElementById('toggleFetch');
 const pausedMessage = document.getElementById('pausedMessage');
+const toggleFitButton = document.getElementById('toggleFit');
+
 
 function toggleFetchOutput() {
     if (isPaused) {
@@ -426,13 +476,33 @@ function toggleFetchOutput() {
     }
     isPaused = !isPaused;
 }
+function toggleFit() {
+    fitWindow = ! fitWindow;
+    if (fitWindow) {
+        toggleFitButton.classList.add('fit-tty');
+        toggleFitButton.setAttribute('title', 'terminal fit tty');
+    } else {
+        toggleFitButton.classList.remove('fit-tty');
+        toggleFitButton.setAttribute('title', 'terminal fit window');
+    }
+    autoFit();
+    viewOutput(currentCommandId);
+}
 
 toggleButton.addEventListener('click', toggleFetchOutput);
+toggleFitButton.addEventListener('click', toggleFit);
+
+document.getElementById('thStatus').addEventListener('click', () => {
+    showRunningOnly = !showRunningOnly;
+    hiddenCommandIds = [];
+    fetchCommands(showRunningOnly);
+});
 
 window.addEventListener('resize', adjustOutputHeight);
-window.addEventListener('load', initResizer);
-
-fetchCommands();
-setInterval(fetchCommands, 5000);
-
+window.addEventListener('load', () => {
+    initResizer();
+    fetchCommands();
+    adjustOutputHeight();
+    setInterval(fetchCommands, 5000);
+});
 
